{-# OPTIONS -fwarn-incomplete-patterns -fwarn-unused-matches -fwarn-name-shadowing #-}

-- | Pretty printing for desugared source.
module DDC.Desugar.Pretty
	(stripAnnot)
where
import DDC.Desugar.Transform
import DDC.Type.Data.Pretty	()
import DDC.Desugar.Exp
import DDC.Main.Error
import DDC.Main.Pretty		
import DDC.Type
import DDC.Var

stage = "Desugar.Pretty"

stripAnnot xx	
 = transformN (\_ -> Nothing :: Maybe ()) xx

annot nn x
 = case nn of
 	Nothing	-> x
	Just n	-> "[" % n % ": " % x % "]"

pprVar_unqual var
 = ppr $ var { varModuleId = ModuleIdNil }


-- Top -------------------------------------------------------------------------
instance Pretty a PMode => Pretty (Top (Maybe a)) PMode where
 ppr xx
  = case xx of
	PImport nn ms
	 -> annot nn
	 	("import {\n"
	 		%> ("\n" %!% (map (\m -> m % ";") ms))
			% "\n}\n\n")

	PExtern nn v tv (Just to)
	 -> let mSeaName	= takeSeaNameOfVar v
		pName		= case mSeaName of  { Nothing -> ppr " "; Just s  -> ppr $ show s }
		
	    in annot nn 
	 	("foreign import " % pName %% v % " " 
			%> ("\n" % ":: " % prettyTypeSplit tv	% "\n"
				 % ":$ " % to			% ";\n"))
		% "\n"

	PExtern nn v tv Nothing
	 -> annot nn 
	 	("extern " % v	
			%> ("\n" % ":: " % prettyTypeSplit tv	% ";\n"))
		% "\n"
		
	-- super sigs
	PSuperSig nn v k
	 -> annot nn
	 	("class " % v %> " :: " % k) % ";\n"

	-- kind sigs
	PKindSig nn v k
	 | resultKind k == kValue	
	 -> annot nn ("data" <> v <> "::" <> k % ";\n\n")

 	 | resultKind k == kEffect
  	 -> annot nn ("effect" <> v <> "::" <> k % "\n\n")

	 | otherwise	
	 -> annot nn ("type" <> v <> "::" <> k % ";\n\n")

	-- types
	PTypeSynonym nn v t
	 -> annot nn 
		("type " <> v <> " = " % t % ";\n\n")

	PRegion nn v
	 -> annot nn
	 	("region " % v) % ";\n"

	PData nn ddef
	 -> annot nn (ppr ddef)

	-- data classes
	PClassDecl nn v ts sigs
	 -> annot nn
	 	("class " % pprVar_unqual v % " " % " " %!% map pprPClassDict_varKind ts % " where\n"
			% "{\n"
			%> ("\n\n" %!% map (\(v', sig) -> pprVar_unqual v' 
					% ("\n        :: " %> prettyTypeSplit sig 	% ";")) sigs)
			% "\n}\n")

	PClassInst nn v ts ss
	 -> annot nn 
	 	("instance " % v % " " % " " %!% map prettyTypeParens ts % " where\n"
			% "{\n"
			%> ("\n\n" %!% ss) % "\n"
			% "}\n\n")

	-- projections
	PProjDict nn t ss
	 -> annot nn
	 	("project " % t % " where\n"
			% "{\n"
			%> ("\n\n" %!% ss) % "\n"
			% "}\n\n")

	PTypeSig nn sigMode v t
	 -> annot nn
	 	(v %> ("\n" % sigMode %% prettyTypeSplit t))	% ";\n\n"

	PBind nn v x
	 -> annot nn 
	 	(v % "\n =      " %> x) % ";\n\n"


pprPClassDict_varKind tt
 = case tt of
	TVar k (UVar v)	-> parens $ v <> "::" <> k
	_		-> panic stage "pprPClassDict_varKind: no match\n"


-- Exp -------------------------------------------------------------------------
instance Pretty a PMode => Pretty (Exp (Maybe a)) PMode where
 ppr xx	
  = case xx of
  	XNil			-> ppr "@XNil"
	XVoid	nn		-> annot nn (ppr "@XVoid")
	XLit	nn l		-> annot nn (ppr l)
	XVar	nn v		-> annot nn (ppr v)
	XVarInst nn v		-> annot nn ("@XVarInst " % v)
	XProj 	nn x j		-> annot nn ("@XProj "  % prettyExpParens x % " " % j)
	XProjT	nn t j		-> annot nn ("@XProjT " % prettyTypeParens t % " " % j)

	XLambda    	nn v x 		
	 -> annot nn ("\\" % v  % " ->\n" % x)

	XLambdaTEC 	nn v x t eff clo 
	 -> annot nn ("\\" % v  % " (" % eff % " " % clo % ") :: " % t % " ->\n" % x)

	XApp	Nothing x1 x2	
	 -> x1 % " " % prettyExpParens x2

	XApp	nn x1 x2	
	 -> annot nn (x1 %> ("\n" % prettyExpParens x2))

	XMatch  nn Nothing aa	
	 -> annot nn 
	 	("match {\n"
		 %> "\n\n" %!% aa
		 % "\n}")


	XMatch  nn (Just x1) aa	
	 -> annot nn 
	 	("match " % x1 % " with {\n"
		 %> "\n\n" %!% aa
		 % "\n}")
	 
	XDo	nn ss
	 -> annot nn 
	 	("do {\n"
		 %> ("\n" %!% ss)
		 % "\n}")
	 
	XIfThenElse nn x1 x2 x3
	 -> annot nn 
	 	("if " % x1 % "\n" %> (" then " % x2) % "\n" %> (" else " % x3))

	XProjTagged 	nn vI tC x j	
	 -> annot nn ("@XProjTagged " % vI % " " % tC % " " % prettyExpParens x % " " % j)

	XProjTaggedT 	nn vI tC j
	 -> annot nn ("@XProjTaggedT " % vI % " " % tC % j)


-- | Pretty print an expression, wrapping it parens if it's not atomic.
prettyExpParens :: Pretty (Exp a) PMode => Exp a -> Str 
prettyExpParens xx
 = case xx of
	XLambda{}	-> "\n" %> ("(" % xx % ")")
	XVar{}		-> ppr xx
	_		-> "(" % xx % ")"


-- Proj ------------------------------------------------------------------------
instance Pretty a PMode => Pretty (Proj (Maybe a)) PMode where
 ppr xx
  = case xx of
  	JField  nn v	-> annot nn ("." % v)
	JFieldR nn v	-> annot nn ("#" % v)
	

-- Stmt ------------------------------------------------------------------------
instance Pretty a PMode => Pretty (Stmt (Maybe a)) PMode where
 ppr xx
  = case xx of
	SBind nn Nothing x
	 -> annot nn
	 	(ppr x) % ";"

  	SBind nn (Just v) x@XLambda{}		
	 -> let v'	= v { varModuleId = ModuleIdNil}
	    in  annot nn 
	  	 	(v' %>> "\n =      " %> x) % ";\n"
	 
	SBind nn (Just v) x
	 -> let v'	= v { varModuleId = ModuleIdNil}
	    in  annot nn
	 	 	(v' %>> " = " % x)	% ";"

	SBindMonadic nn pat x
	 -> annot nn
	    		(pat %>> " <- " % x)	% ";"
	
	SBindPat nn pat x
	 -> annot nn	(pat %>> " = " % x)	% ";"
	
	SSig  nn sigMode vs  t	
	 -> let vs'	= map (\v -> v { varModuleId = ModuleIdNil}) vs
	    in	annot nn (vs' %!% ", " %>> (ppr sigMode %% t)) % ";"


-- Alt -------------------------------------------------------------------------
instance Pretty a PMode => Pretty (Alt (Maybe a)) PMode where
 ppr xx
  = case xx of
	AAlt	 nn [] x	
	 -> annot nn ("\\= " % x) % ";"

	AAlt	 nn gs x	
	 -> annot nn ("|"  % "\n," %!% gs % "\n= " % x) % ";"


-- Guard -----------------------------------------------------------------------
instance Pretty a PMode => Pretty (Guard (Maybe a)) PMode where
 ppr gg
  = case gg of
  	GCase nn pat	-> annot nn ("| " % pat)
	GExp  nn pat x	-> annot nn (" "  % pat %> " <- " % x)
	

-- Pat ------------------------------------------------------------------------
instance Pretty a PMode => Pretty (Pat (Maybe a)) PMode where
 ppr ww
  = case ww of
	WConLabel nn v lvs	
		-> annot nn (
			v % " {" % 
				", " %!% (map (\(l, v') -> l % " = " % v') lvs) % 
			"}")

	WLit nn l 	-> annot nn (ppr l)
	WVar nn v 	-> annot nn (ppr v)
	WAt nn v w	-> annot nn (v % "@(" % w % ")")

	WConLabelP nn v lvs	
		-> annot nn (
			v % " (--pat--){" % 
				", " %!% (map (\(l, v') -> l % " = " % v') lvs) % 
			"}")


-- Label -----------------------------------------------------------------------
instance Pretty a PMode => Pretty (Label (Maybe a)) PMode where
 ppr ll
  = case ll of
  	LIndex	nn i		-> annot nn ("." % i)
	LVar	nn v		-> annot nn ("." % v)
	
